function [ dbCA, cones, objsize, dir, Q, P, esrc, edst ] = eliminate( prob, destructive )
if nargin < 2, destructive = false; end

global cvx___
[ dbCA, cones, objsize, dir, Q, P, esrc, edst ] = extract( prob, destructive );

if ~issparse( dbCA ),
    dbCA = sparse( dbCA );
end
nn = size( dbCA, 1 );
rsv = sparse( 1, 1, 1, nn, 1 );
for k = 1 : length( cones ),
    rsv = rsv + sparse( cones( k ).indices, 1, 1, nn, 1 );
end
nobj     = prod( objsize );
rsv      = int32( full( rsv ) );
ndxs     = [ 1 : nn ]';
nold     = nn;
ondxs    = ndxs;
iter     = 0;
reduced  = 1;

if size( dbCA, 1 ) == nobj,
    return
end

%
% Eliminate variables that are completely unused
%

rows = ~( any( dbCA, 2 ) | double( rsv ) );
if any( rows ),
    dbCA( rows, : ) = [];
    Q( :, rows ) = [];
    rsv( rows )  = [];
    ndxs( rows ) = [];
    nn = size( dbCA, 1 );
end

while 1,
    iter = iter + 1;
    if reduced == iter - 2,
        break;
    end
    if rem( iter, 2 ) == 1,
        [ xR, dbCA ] = cvx_bcompress( dbCA, 'full', nobj );
        if size( xR, 1 ) ~= size( xR, 2 ),
            P = P * cvx_invert_structure( xR );
            reduced = iter;
        end
    else
        while ~isempty( dbCA ),

            %
            % Select columns to eliminate. More about this later. In short,
            % we're looking for an invertible diagonal block of A whose
            % rows and columns are sparse, so that no fill-in occurs.
            %

            [ rows, cols ] = cvx_eliminate_mex( dbCA, nobj, rsv );
            if ~any( rows ), break; end
            rows = rows ~= 0;
            cols = cols ~= 0;
            rowX = ~rows;
            colX = ~cols;

            %
            % [ x1^T x2^T ] [ C1 A11 A12 ] [ 1  ]
            %               [ C2 A21 A22 ] [ y1 ] = 0
            %                              [ y2 ]
            %
            % [ x1^T x2^T ] = x1^T [ I -A12*A22i ]
            %
            % [ G Y1^T Y2^T ] = [ G Y1^T ] [ I  0  -C2'*A22i'  ]
            %                              [ 0  I  -A21'*A22i' ]
            %

            A11  = dbCA( rowX, colX );
            A12  = dbCA( rowX, cols );
            A21  = dbCA( rows, colX );
            A22  = dbCA( rows, cols );
            if ( size( A22, 1 ) ~= size( A22, 2 ) | nnz( A22 ) ~= size( A22, 1 ) ),
                error( sprintf( 'There seems to be an error in the CVX presolver routine.\nPlease report this to the authors; and if possible, include the\ncvx model and data that gave you this error.' ) );
            end
            [ ii, jj, vv ] = find( A22 );
            A22i = sparse( jj, ii, 1.0 ./ vv );
            temp = - A22i * A21;
            P    = P( :, colX ) + P( :, cols ) * temp;
            temp = - A12 * A22i;
            Q    = Q( :, rowX ) + Q( :, rows ) * temp';
            dbCA = A11 + temp * A21;
            rsv  =  rsv( rowX );
            ndxs = ndxs( rowX );
            reduced = iter;
        end
    end
end

%
% The first nobj columns of P need to be negated in order to produce the right
% numerical results. This is basically because the dcbA matrix should have
% negated the corresponding rows, but we didn't want to. (If you express the
% Lagrangian in terms of the block dcbA matrix this becomes clear.)
%

if ~isempty( P ),
    P( :, 1 : nobj ) = - P( :, 1 : nobj );
end

%
% Move the cone indices to their new locations
%

ndxs = full( sparse( ndxs, 1, 1 : length( ndxs ), nold, 1 ) );
for k = 1 : length( cones ),
    cones(k).indices = reshape( ndxs(cones(k).indices), size(cones(k).indices) );
end
esrc = ndxs( esrc );
edst = ndxs( edst );
if ~isempty( esrc ),
    tt = esrc == 0 | edst == 0;
    esrc( tt ) = [];
    edst( tt ) = [];
end

% Copyright 2005 Michael C. Grant and Stephen P. Boyd.
% See the file COPYING.txt for full copyright information.
% The command 'cvx_where' will show where this file is located.
