function solve( prob, quiet )
if nargin < 2, quiet = false; end

global cvx___
p = index( prob );
[ At, cones, objsize, sgn, Q, P, esrc ] = eliminate( prob );
nobj = prod( objsize );

if nobj == 0,
    error( 'Shouldn''t have an empty objective here.' );
elseif nobj > 1 & ~cvx___.problems( p ).separable,
    error( 'Non-separable multiobjective problems are not supported.' );
elseif ~isempty( esrc ),
    error( 'Non-GP exp() and log() constraints are not supported.' );
end

c = At( :, 1 : nobj );
At( :, 1 : nobj ) = [];
d = c( 1, : );
c( 1, : ) = [];
[ n1, m ] = size( At );
if n1 < 1,
    b = zeros( m, 1 );
else
    b = - At( 1, : ).';
    At( 1, : ) = [];
end
n = n1 - 1;
lambda = ones( nobj, 1 );
for k = 1 : length( cones ),
    cones(k).indices = cones(k).indices - 1;
end

%
% Ferret out the degenerate and overdetermined problems
%

found = true;
if m > n & n > 0,
    %
    % Too many equality constraints
    %
    x      = NaN * ones( n, 1 );
    y      = NaN * ones( m, 1 );
    value  = NaN * ones( objsize );
    status = 'Overdetermined';
    estr = sprintf( 'Overdetermined equality constraints detected.\n   CVX cannot solve this problem; but it is likely infeasible.' );
    if ~quiet,
        disp( estr );
    else
        warning( estr );
    end
    pval = NaN;
    dval = NaN;
elseif n == 0,
    %
    % No variables
    %
    if any( b ~= 0 ),
        if ~quiet,
            disp( 'Infeasible problem; solution determined analytically.' );
        end
        status = 'Infeasible';
        x = NaN * ones( n, 1 );
        y = - b / ( b' * b );
        value = sgn * Inf * ones( objsize );
        pval = NaN;
        dval = 0;
    else
        if ~quiet,
            disp( 'Solution determined analytically.' );
        end
        status = 'Solved';
        x = zeros( n, 1 );
        y = zeros( m, 1 );
        value = sgn * reshape( d, objsize );
        pval = 1;
        dval = 1;
    end
elseif all( b == 0 ) & all( c( : ) == 0 ),
    %
    % Homogeneous feasibility problem
    %
    if ~quiet,
        disp( 'Solution determined analytically.' );
    end
    status = 'Solved';
    x = zeros( n, 1 );
    y = zeros( m, 1 );
    value = 0;
    pval = 1;
    dval = 1;
else
    tt = ~any( At, 1 );
    if any( tt ) & all( b( tt ) ~= 0 ),
        %
        % Trivially infeasible problem
        %
        if ~quiet,
            disp( 'Infeasible problem; solution determined analytically.' );
        end
        status = 'Infeasible';
        value = sgn * Inf * ones( objsize );
        x = NaN * ones( n, 1 );
        y = sgn * b .* tt( : );
        y = y / abs( b' * y );
        pval = NaN;
        dval = 0;
    else
        tt = ~any( At, 2 );
        if any( tt ),
            for k = 1 : length( cones ),
                tt( cones(k).indices ) = false;
            end
            if any( tt ) & all( c( tt ) ~= 0 ),
                %
                % Trivially unbounded problem
                %
                if ~quiet,
                    disp( 'Unbounded problem; solution determined analytically.' );
                end
                status = 'Unbounded';
                value = -sgn * Inf * ones( objsize );
                x = -sgn * c .* tt( : );
                x = x / abs( c' * x );
                y = NaN * ones( m, 1 );
                pval = 0;
                dval = NaN;
            else
                found = false;
            end
        else
            found = false;
        end
    end
end

%
% Call the solver
%

if ~found,
    if m == 0,
        if ~quiet,
            disp( 'Degenerate problem encountered; an unbounded direction search will be' );
            disp( 'performed. The solver''s status message should be ignored, as it will' );
            disp( 'not coincide with cvx_status.' );
        end
        [ x, y, status ] = cvx_solve_sedumi( c * lambda, -sgn, zeros( n, 1 ), 1, cones, quiet, cvx___.problems( p ).precision );
        y = zeros( m, 1 );
        switch status,
            case { 'Solved', 'Inaccurate/Solved' },
                status = 'Unbounded';
                value = -sgn * Inf * ones( objsize );
                pval = 0;
                dval = NaN;
            case { 'Infeasible', 'Inaccurate/Infeasible' },
                status = 'Solved';
                value = sgn * reshape( d, objsize );
                x = zeros( n, 1 );
                pval = 1;
                dval = 1;
            otherwise,
                value = NaN * ones( objsize );
                x = NaN * ones( n, 1 );
                pval = NaN;
                dval = NaN;
        end
    else
        [ x, y, status ] = cvx_solve_sedumi( At, b, c * lambda, sgn, cones, quiet, cvx___.problems( p ).precision );
        switch status,
        case { 'Solved', 'Inaccurate/Solved' },
            value = sgn * reshape( c' * x + d', objsize );
            pval = 1;
            dval = 1;
        case { 'Infeasible', 'Inaccurate/Infeasible' },
            value = sgn * Inf * ones( objsize );
            pval = NaN;
            dval = 0;
        case { 'Unbounded', 'Inaccurate/Unbounded' },
            value = -sgn * Inf * ones( objsize );
            pval = 0;
            dval = NaN;
        otherwise,
            value = NaN * ones( objsize );
            pval = NaN;
            dval = NaN;
        end
    end
end

gvec = cvx___.problems( p ).geometric;
if nnz( gvec ),
    value( gvec ) = exp( value( gvec ) );
end
value = full( value );
if ~quiet,
    disp( '------------------------------------------------------------------------' );
    disp( sprintf( 'Status: %s', status ) );
    if length( value ) == 1,
        disp( sprintf( 'Optimal value (cvx_optval): %+g', value ) );
    else
        disp( sprintf( 'Optimal value (cvx_optval): (multiobjective)' ) );
    end
end

%
% Push the results into the master CVX workspace
%

global cvx___
cvx___.x = full( Q * [ pval ; x ] );
cvx___.y = full( P * [ dval * lambda ; y ] );
cvx___.problems( p ).result = value;
cvx___.problems( p ).status = status;
if nnz( cvx___.exponential ),
    esrc = find( cvx___.exponential );
    edst = cvx___.exponential( esrc );
    cvx___.x( edst ) = exp( cvx___.x( esrc ) );
end

% Copyright 2005 Michael C. Grant and Stephen P. Boyd.
% See the file COPYING.txt for full copyright information.
% The command 'cvx_where' will show where this file is located.
